@@ -1,66 +1,57 @@
-use crate::{
-    configuration::Handler,
-    filesystems::{
-        DirItemInfo,
-        FileInfo,
-        FilesystemErrors,
-    },
-    handlers::{
-        Messages,
-        TransportHandler,
-    },
-    Configuration,
+use crate::configuration::Handler;
+use crate::handlers::TransportHandler;
+use crate::Configuration;
+use gveditor_core_api::extensions::base::ExtensionInfo;
+use gveditor_core_api::filesystems::{
+    DirItemInfo,
+    FileInfo,
+    FilesystemErrors,
+};
+use gveditor_core_api::messaging::{
+    ExtensionMessages,
+    Messages,
+};
+use gveditor_core_api::state::StatesList;
+use gveditor_core_api::{
+    Errors,
     State,
-    StatesList,
 };
 use jsonrpc_derive::rpc;
-use serde::{
-    Deserialize,
-    Serialize,
-};
+
 use std::sync::{
     Arc,
     Mutex,
 };
-use tokio::sync::{
-    mpsc::{
-        channel,
-        Receiver,
-        Sender,
-    },
-    Mutex as AsyncMutex,
-};
+use tokio::sync::mpsc::Receiver;
+use tokio::sync::Mutex as AsyncMutex;
 
 pub struct Server {
     states: Arc<Mutex<StatesList>>,
     config: Configuration,
-    sender: Arc<Mutex<Sender<Messages>>>,
 }
 
 impl Server {
     /// Create a new Server
     pub fn new(config: Configuration, states: Arc<Mutex<StatesList>>) -> Self {
-        let (sender, receiver) = channel::<Messages>(1);
-        let sender = Arc::new(Mutex::new(sender));
+        Self::create_receiver(
+            states.clone(),
+            config.receiver.clone(),
+            config.handler.clone(),
+        );
 
-        Self::create_receiver(states.clone(), receiver, config.handler.clone());
-
-        Self {
-            config,
-            states,
-            sender,
-        }
+        Self { config, states }
     }
 
     /// Receive all incoming messages
     pub fn create_receiver(
         states: Arc<Mutex<StatesList>>,
-        mut receiver: Receiver<Messages>,
+        receiver: Arc<AsyncMutex<Receiver<Messages>>>,
         handler: Handler,
     ) {
         std::thread::spawn(move || {
             let rt = tokio::runtime::Runtime::new().unwrap();
             rt.block_on(async move {
+                let mut receiver = receiver.lock().await;
                 loop {
                     if let Some(message) = receiver.recv().await {
                         Self::process_message(states.clone(), message, handler.clone()).await;
@@ -75,7 +66,9 @@ impl Server {
         let states = self.states.clone();
         let mut handler = self.config.handler.lock().await;
 
-        handler.run(states.clone(), self.sender.clone()).await;
+        handler
+            .run(states.clone(), self.config.sender.clone())
+            .await;
     }
 
     /// Process every message
@@ -101,21 +94,23 @@ impl Server {
                         state: state.lock().unwrap().to_owned(),
                     };
                     handler.send(message).await;
+
+                    state.lock().unwrap().run_extensions().await;
                 }
             }
-            Messages::StateUpdated { .. } => {}
+            Messages::StateUpdated { .. } => {
+                let states = states.lock().unwrap();
+                states.notify_extensions(ExtensionMessages::CoreMessage(msg));
+            }
+            _ => {
+                // Forward to the handler messages not handled here
+                let handler = handler.lock().await;
+                handler.send(msg).await;
+            }
         }
     }
 }
 
-/// Global errors enum
-#[derive(Serialize, Deserialize, Debug)]
-pub enum Errors {
-    StateNotFound,
-    Fs(FilesystemErrors),
-    BadToken,
-}
-
 pub type RPCResult<T> = jsonrpc_core::Result<T>;
 
 /// Definition of all JSON RPC Methods
@@ -144,6 +139,21 @@ pub trait RpcMethods {
         state_id: u8,
         token: String,
     ) -> RPCResult<Result<Vec<DirItemInfo>, Errors>>;
+
+    #[rpc(name = "get_ext_info_by_id")]
+    fn get_ext_info_by_id(
+        &self,
+        extension_id: String,
+        state_id: u8,
+        token: String,
+    ) -> RPCResult<Result<ExtensionInfo, Errors>>;
+
+    #[rpc(name = "get_ext_list_by_id")]
+    fn get_ext_list_by_id(
+        &self,
+        state_id: u8,
+        token: String,
+    ) -> RPCResult<Result<Vec<String>, Errors>>;
 }
 
 /// JSON RPC manager
@@ -186,11 +196,18 @@ impl RpcMethods for RpcManager {
         let states = self.states.lock().unwrap();
         // Try to get the requested state
         if let Some(state) = states.get_state_by_id(state_id) {
+            let state = state.lock().unwrap();
             // Make sure the token is valid
-            if state.lock().unwrap().has_token(&token) {
+            if state.has_token(&token) {
                 // Try to get the requested filesystem implementation
-                if let Some(filesystem) = state.lock().unwrap().get_fs_by_name(&filesystem_name) {
-                    Ok(filesystem.lock().unwrap().read_file_by_path(&path))
+                if let Some(filesystem) = state.get_fs_by_name(&filesystem_name) {
+                    let result = filesystem.lock().unwrap().read_file_by_path(&path);
+                    state.notify_extensions(ExtensionMessages::ReadFile(
+                        state_id,
+                        filesystem_name,
+                        result.clone(),
+                    ));
+                    Ok(result)
                 } else {
                     Ok(Err(Errors::Fs(FilesystemErrors::FilesystemNotFound)))
                 }
@@ -214,11 +231,19 @@ impl RpcMethods for RpcManager {
         let states = self.states.lock().unwrap();
         // Try to get the requested state
         if let Some(state) = states.get_state_by_id(state_id) {
+            let state = state.lock().unwrap();
             // Make sure the token is valid
-            if state.lock().unwrap().has_token(&token) {
+            if state.has_token(&token) {
                 // Try to get the requested filesystem implementation
-                if let Some(filesystem) = state.lock().unwrap().get_fs_by_name(&filesystem_name) {
-                    Ok(filesystem.lock().unwrap().list_dir_by_path(&path))
+                if let Some(filesystem) = state.get_fs_by_name(&filesystem_name) {
+                    let result = filesystem.lock().unwrap().list_dir_by_path(&path);
+                    state.notify_extensions(ExtensionMessages::ListDir(
+                        state_id,
+                        filesystem_name,
+                        path,
+                        result.clone(),
+                    ));
+                    Ok(result)
                 } else {
                     Ok(Err(Errors::Fs(FilesystemErrors::FilesystemNotFound)))
                 }
@@ -229,4 +254,48 @@ impl RpcMethods for RpcManager {
             Ok(Err(Errors::StateNotFound))
         }
     }
+
+    /// Returns the information about a extension
+    fn get_ext_info_by_id(
+        &self,
+        extension_id: String,
+        state_id: u8,
+        token: String,
+    ) -> RPCResult<Result<ExtensionInfo, Errors>> {
+        let states = self.states.lock().unwrap();
+        // Try to get the requested state
+        if let Some(state) = states.get_state_by_id(state_id) {
+            let state = state.lock().unwrap();
+            // Make sure the token is valid
+            if state.has_token(&token) {
+                // Try to get the requested info about the extension
+                Ok(state.get_ext_info_by_id(&extension_id))
+            } else {
+                Ok(Err(Errors::BadToken))
+            }
+        } else {
+            Ok(Err(Errors::StateNotFound))
+        }
+    }
+    /// Returns the list of extensions in the specified state
+    fn get_ext_list_by_id(
+        &self,
+        state_id: u8,
+        token: String,
+    ) -> RPCResult<Result<Vec<String>, Errors>> {
+        let states = self.states.lock().unwrap();
+        // Try to get the requested state
+        if let Some(state) = states.get_state_by_id(state_id) {
+            let state = state.lock().unwrap();
+            // Make sure the token is valid
+            if state.has_token(&token) {
+                // Try to get the requested info about the extension
+                Ok(Ok(state.get_ext_list_by_id()))
+            } else {
+                Ok(Err(Errors::BadToken))
+            }
+        } else {
+            Ok(Err(Errors::StateNotFound))
+        }
+    }
 }
